==> easy_validation-0.0.1.tar.gz: lib/model_state.dart <==
import 'package:easy_validation/annotations/validation_annotation.dart';
import 'package:easy_validation/annotations/easy_validator.dart';
import 'package:easy_validation/annotations/validation_error.dart';
import 'package:reflectable/reflectable.dart';

/// Encapsulates the state of model binding or one of those properties.
/// {@category ModelState}
class ModelState {
  static List<ValidationError> _errors;

  /// Gets the state of current [model] binding of [TModel] type.
  /// Browses all declared metadata of [ValidationAnnotation] type, for each property into your class model.
  /// Then, executes "IsValid" method that returns true or false. For each property, first of them that returns false invalidates your property, and so, your model. If there is none, your property and model are valid.
  static bool isValid<TModel>(
    TModel model,
  ) {
    try {
      InstanceMirror instanceMirror = easyValidator.reflect(model);
      ClassMirror classMirror = easyValidator.reflectType(TModel);
      _errors = new List<ValidationError>();

      classMirror.declarations
          .forEach((String propertyName, DeclarationMirror declarationMirror) {
        ValidationError error = _validateProperty(
          propertyName,
          declarationMirror,
          instanceMirror,
          model,
        );
        if (error != null) _errors.add(error);
      });

      return _errors.isEmpty;
    } catch (e) {
      print(e);
      return false;
    }
  }

  /// Gets the state of current [propertyName] into the [model] binding of [TModel] type.
  /// Browses all declared metadata of [ValidationAnnotation] type for property.
  /// Then, executes "IsValid" method that returns true or false. First of them that retuns false invalidates your property. If there is none, your property is valid.
  static bool isValidProperty<TModel>(
    TModel model,
    String propertyName,
  ) {
    try {
      InstanceMirror instanceMirror = easyValidator.reflect(model);
      ClassMirror classMirror = easyValidator.reflectType(TModel);
      DeclarationMirror declarationMirror =
          classMirror.declarations[propertyName];
      _errors = new List<ValidationError>();

      if (declarationMirror != null) {
        ValidationError error = _validateProperty(
          propertyName,
          declarationMirror,
          instanceMirror,
          model,
        );
        if (error != null) _errors.add(error);
      } else {
        throw Exception('No property found for this name in this object');
      }

      return _errors.isEmpty;
    } catch (e) {
      print(e);
      return false;
    }
  }

  static ValidationError _validateProperty<TModel>(
    String key,
    DeclarationMirror declarationMirror,
    InstanceMirror instanceMirror,
    TModel model,
  ) {
    ValidationError error;
    List<ValidationAnnotation> annotations =
        _getValidators(declarationMirror.metadata);

    for (ValidationAnnotation annotation in annotations) {
      Object value = instanceMirror.invokeGetter(key);
      bool isValid = annotation.isValid(value, model);
      if (!isValid) {
        error = ValidationError(
          propertyName: key,
          validatorType: annotation.runtimeType,
          error: annotation.error,
        );
        break;
      }
    }

    return error;
  }

  static List<ValidationAnnotation> _getValidators(List<Object> metadata) {
    List<ValidationAnnotation> validators = new List<ValidationAnnotation>();
    for (Object metadatum in metadata)
      if (metadatum is ValidationAnnotation) validators.add(metadatum);
    validators.sort((a, b) => a.criticityLevel.compareTo(b.criticityLevel));
    return validators;
  }

  /// Gets all errors that EasyValidation has detected.
  static List<ValidationError> get errors {
    return ModelState._errors;
  }
}

==> easy_validation-0.0.2.tar.gz: lib/model_state.dart <==
import 'package:easy_validation/annotations/validation_annotation.dart';
import 'package:easy_validation/annotations/easy_validator.dart';
import 'package:easy_validation/annotations/validation_error.dart';
import 'package:reflectable/reflectable.dart';

/// Encapsulates the state of model binding or one of those properties.
/// {@category ModelState}
class ModelState {
  static Map<String, ValidationError> _errors;

  /// Gets the state of current [model] binding of [TModel] type.
  /// Browses all declared metadata of [ValidationAnnotation] type, for each property into your class model.
  /// Then, executes "IsValid" method that returns true or false. For each property, first of them that returns false invalidates your property, and so, your model. If there is none, your property and model are valid.
  static bool isValid<TModel>(
    TModel model,
  ) {
    try {
      InstanceMirror instanceMirror = easyValidator.reflect(model);
      ClassMirror classMirror = easyValidator.reflectType(TModel);
      _errors = Map<String, ValidationError>();

      classMirror.declarations
          .forEach((String propertyName, DeclarationMirror declarationMirror) {
        ValidationError error = _validateProperty(
          propertyName,
          declarationMirror,
          instanceMirror,
          model,
        );
        if (error != null)
          _errors[propertyName] = error;
      });

      return _errors.isEmpty;
    } catch (e) {
      print(e);
      return false;
    }
  }

  /// Gets the state of current [propertyName] into the [model] binding of [TModel] type.
  /// Browses all declared metadata of [ValidationAnnotation] type for property.
  /// Then, executes "IsValid" method that returns true or false. First of them that retuns false invalidates your property. If there is none, your property is valid.
  static bool isValidProperty<TModel>(
    TModel model,
    String propertyName,
  ) {
    try {
      InstanceMirror instanceMirror = easyValidator.reflect(model);
      ClassMirror classMirror = easyValidator.reflectType(TModel);
      DeclarationMirror declarationMirror =
          classMirror.declarations[propertyName];
      _errors = Map<String, ValidationError>();

      if (declarationMirror != null) {
        ValidationError error = _validateProperty(
          propertyName,
          declarationMirror,
          instanceMirror,
          model,
        );
        if (error != null)
          _errors[propertyName] = error;
      } else {
        throw Exception('No property found for this name in this object');
      }

      return _errors.isEmpty;
    } catch (e) {
      print(e);
      return false;
    }
  }

  static ValidationError _validateProperty<TModel>(
    String key,
    DeclarationMirror declarationMirror,
    InstanceMirror instanceMirror,
    TModel model,
  ) {
    ValidationError error;
    List<ValidationAnnotation> annotations =
        _getValidators(declarationMirror.metadata);

    for (ValidationAnnotation annotation in annotations) {
      Object value = instanceMirror.invokeGetter(key);
      bool isValid = annotation.isValid(value, model);
      if (!isValid) {
        error = ValidationError(
          propertyName: key,
          validatorType: annotation.runtimeType,
          error: annotation.error,
        );
        break;
      }
    }

    return error;
  }

  static List<ValidationAnnotation> _getValidators(List<Object> metadata) {
    List<ValidationAnnotation> validators = new List<ValidationAnnotation>();
    for (Object metadatum in metadata)
      if (metadatum is ValidationAnnotation) validators.add(metadatum);
    validators.sort((a, b) => a.criticityLevel.compareTo(b.criticityLevel));
    return validators;
  }

  /// Gets all errors that EasyValidation has detected.
  static Map<String, ValidationError> get errors {
    return ModelState._errors;
  }
}
